<?php
namespace Elementor;

class WYBMV_Widget extends Widget_Base {
    public function get_style_depends() {
        wp_enqueue_style( 'wybmv-custom', plugins_url( 'assets/css/custom.css', __FILE__ ));
        return [
            'wybmv-custom',
        ];
    }
    public function get_name() {
        return 'wybmv-forms';
    }
    public function get_title() {
        return esc_html__('Will You Be My Valentine', 'wybmv');
    }
    
    public function get_icon() {
        return 'eicon-form-horizontal';
    }
    
    public function get_categories() {
        return [ 'modeltheme-widgets' ];
    }
    
    protected function register_controls() {

        $this->start_controls_section(
            'section_title',
            [
                'label' => esc_html__( 'General Options', 'wybmv' ),
            ]
        );
        $this->add_control(
            'shortcode_alignment',
            [
                'label' => esc_html__( 'Alignment', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'wybmv' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'centered' => [
                        'title' => esc_html__( 'Center', 'wybmv' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'wybmv' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'default' => 'centered',
                'toggle' => true,
            ]
        );
        $this->add_control(
            'modal_width',
            [
                'label' => esc_html__( 'Modal Width', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::SELECT,
                'default' => '30%',
                'options' => [
                    '10%' => esc_html__( '10%', 'wybmv' ),
                    '20%' => esc_html__( '20%', 'wybmv' ),
                    '30%' => esc_html__( '30%', 'wybmv' ),
                    '40%' => esc_html__( '40%', 'wybmv' ),
                    '50%' => esc_html__( '50%', 'wybmv' ),
                    '60%' => esc_html__( '60%', 'wybmv' ),
                    '70%' => esc_html__( '70%', 'wybmv' ),
                    '80%' => esc_html__( '80%', 'wybmv' ),
                    '90%' => esc_html__( '90%', 'wybmv' ),
                    '100%' => esc_html__( '100%', 'wybmv' ),
                ],
                'selectors' => [
                    '{{WRAPPER}} .wybmv-modal-width' => 'width: {{VALUE}};',
                ],
            ]
        );
        $this->end_controls_section();
        $this->start_controls_section(
            'section_title_subtitle',
            [
                'label' => esc_html__( 'Title & Subtitle', 'wybmv' ),
            ]
        );
        $this->add_control(
            'title',
            [
                'label' => esc_html__( 'Title', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::TEXT,
                'default' => esc_html__( 'Will you be My Valentine?', 'wybmv' ),
                'placeholder' => esc_html__( 'Type your title here', 'wybmv' ),
            ]
        );
        $this->add_control(
            'show_subtitle',
            [
                'label' => esc_html__( 'Show Subtitle', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Show', 'wybmv' ),
                'label_off' => esc_html__( 'Hide', 'wybmv' ),
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );
        $this->add_control(
            'subtitle',
            [
                'label' => esc_html__( 'Subtitle', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::TEXTAREA,
                'default' => esc_html__( 'Subtitle text', 'wybmv' ),
                'placeholder' => esc_html__( 'Type your subtitle here', 'wybmv' ),
                'condition' => [
                    'show_subtitle' => 'yes',
                ],
            ]
        );
        $this->add_group_control(
            \Elementor\Group_Control_Typography::get_type(),
            [
                'label' => esc_html__( 'Title Typography', 'wybmv' ),
                'name' => 'title_typography',
                'selector' => '{{WRAPPER}} .wybmv-title',
            ]
        );
        $this->add_control(
            'title_color',
            [
                'label' => esc_html__( 'Title Color', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::COLOR,
                'default' => esc_html__( '#000', 'wybmv' ),
                'selectors' => [
                    '{{WRAPPER}} .wybmv-title' => 'color: {{VALUE}}',
                ],
            ]
        );
         $this->add_group_control(
            \Elementor\Group_Control_Typography::get_type(),
            [
                'label' => esc_html__( 'Subtitle Typography', 'wybmv' ),
                'name' => 'subtitle_typography',
                'selector' => '{{WRAPPER}} .wybmv-subtitle',
                'condition' => [
                    'show_subtitle' => 'yes',
                ],
            ]
        );
        $this->add_control(
            'subtitle_color',
            [
                'label' => esc_html__( 'Subtitle Color', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::COLOR,
                'default' => esc_html__( '#000', 'wybmv' ),
                'selectors' => [
                    '{{WRAPPER}} .wybmv-subtitle' => 'color: {{VALUE}}',
                ],
                  'condition' => [
                    'show_subtitle' => 'yes',
                ],
            ]
        ); 
        $this->end_controls_section();
        $this->start_controls_section(
            'section_buttons',
            [
                'label' => esc_html__( 'Buttons', 'wybmv' ),
            ]
        );
        $this->add_control(
            'button_yes',
            [
                'label' => esc_html__( 'Button Text - Yes', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::TEXT,
                'default' => esc_html__( 'Yes', 'wybmv' ),
                'placeholder' => esc_html__( 'Type your text here', 'wybmv' ),
            ]
        );
        $this->add_control(
            'button_no',
            [
                'label' => esc_html__( 'Button Text - No', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::TEXT,
                'default' => esc_html__( 'No', 'wybmv' ),
                'placeholder' => esc_html__( 'Type your text here', 'wybmv' ),
            ]
        );
             $this->add_group_control(
            \Elementor\Group_Control_Typography::get_type(),
            [
                'label' => esc_html__( 'Button Typography', 'wybmv' ),
                'name' => 'button_typography',
                'selector' => '{{WRAPPER}} .wybmv-button',
            ]
        );
            $this->add_control(
            'button_txt_color_yes',
            [
                'label' => esc_html__( 'Button Text Color - Yes', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::COLOR,
                'default' => esc_html__( '#000', 'wybmv' ),
                'selectors' => [
                    '{{WRAPPER}} .wybmv-button.wybmv-yes' => 'color: {{VALUE}}!important',
                ],
            ]
        );
        $this->add_control(
            'button_txt_color_no',
            [
                'label' => esc_html__( 'Button Text Color - No', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::COLOR,
                'default' => esc_html__( '#000', 'wybmv' ),
                'selectors' => [
                    '{{WRAPPER}} .wybmv-button.wybmv-no' => 'color: {{VALUE}}!important',
                ],
            ]
        );
        $this->add_control(
            'button_yes_bg_color',
            [
                'label' => esc_html__( 'Button Backgorund Color - Yes', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::COLOR,
                'default' => esc_html__( '#12D890', 'wybmv' ),
                'selectors' => [
                    '{{WRAPPER}} .wybmv-button.wybmv-yes' => 'background-color: {{VALUE}}',
                ],
            ]
        );
        $this->add_control(
            'button_no_bg_color',
            [
                'label' => esc_html__( 'Button Backgorund Color - No', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::COLOR,
                'default' => esc_html__( '#D8124B', 'wybmv' ),
                'selectors' => [
                    '{{WRAPPER}} .wybmv-button.wybmv-no' => 'background-color: {{VALUE}}',
                ],
            ]
        );
        $this->end_controls_section();
        $this->start_controls_section(
            'section_modal_yes',
            [
                'label' => esc_html__( 'Modal - Yes', 'wybmv' ),
            ]
        );
        $this->add_control(
            'modal_title_yes',
            [
                'label' => esc_html__( 'Modal Title - Yes', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::TEXT,
                'default' => esc_html__( 'Title', 'wybmv' ),
                'placeholder' => esc_html__( 'Type your title here', 'wybmv' ),
            ]
        );
        $this->add_control(
            'modal_paragraph_yes',
            [
                'label' => esc_html__( 'Modal Paragraph - Yes', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::TEXTAREA,
                'default' => esc_html__( 'Paragraph Text', 'wybmv' ),
                'placeholder' => esc_html__( 'Type your text here', 'wybmv' ),
            ]
        );
        $this->add_control(
            'image_yes',
            [
                'label' => esc_html__( 'Choose Image - Yes', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::MEDIA,
                'default' => [
                    'url' => \Elementor\Utils::get_placeholder_image_src(),
                ],
            ]
        );
        $this->add_control(
            'modal_yes_btn_switch',
            [
                'label' => esc_html__( 'Button Type', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::SELECT,
                'default' => 'call',
                'options' => [
                    'call' => esc_html__( 'Phone', 'wybmv' ),
                    'mail' => esc_html__( 'Mail', 'wybmv' ),
                ]
            ]
        );
        $this->add_control(
            'modal_button_yes_phone',
            [
                'label' => esc_html__( 'Add Recipient Phone Number', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::TEXT,
                'default' => esc_html__( '+40(261)713454', 'wybmv' ),
                'placeholder' => esc_html__( 'Add Phone Number', 'wybmv' ),
                'condition' => [
                    'modal_yes_btn_switch' => 'call',
                ],
            ]
        );
        $this->add_control(
        'modal_button_yes_mail',
            [
                'label' => esc_html__( 'Add Recipient Mail', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::TEXT,
                'default' => esc_html__( 'no-reply@test.com', 'wybmv' ),
                'placeholder' => esc_html__( 'Add Recipient Mail', 'wybmv' ),
                'condition' => [
                    'modal_yes_btn_switch' => 'mail',
                ],
            ]
        );
        $this->add_control(
        'modal_button_yes_subject',
            [
                'label' => esc_html__( 'Add Recipient Mail Subject', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::TEXT,
                'default' => esc_html__( 'Will You Be My Valentine', 'wybmv' ),
                'placeholder' => esc_html__( 'Add Mail Subject', 'wybmv' ),
                'condition' => [
                    'modal_yes_btn_switch' => 'mail',
                ],
            ]
        );
        $this->add_control(
        'modal_button_yes_body',
            [
                'label' => esc_html__( 'Add Recipient Mail Message', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::TEXTAREA,
                'default' => esc_html__( 'Roses are red, WiFi is fast, my love for you is as endless as my browser history.', 'wybmv' ),
                'placeholder' => esc_html__( 'Add Mail Message', 'wybmv' ),
                'condition' => [
                    'modal_yes_btn_switch' => 'mail',
                ],
            ]
        );
        $this->add_control(
            'modal_button_yes',
            [
                'label' => esc_html__( 'Modal Button Label - Yes', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::TEXT,
                'default' => esc_html__( 'Message me, sweetheart!', 'wybmv' ),
                'placeholder' => esc_html__( 'Type your button label', 'wybmv' ),
            ]
        );
        $this->end_controls_section();
        $this->start_controls_section(
            'section_modal_no',
            [
                'label' => esc_html__( 'Modal - No', 'wybmv' ),
            ]
        );
        $this->add_control(
            'modal_title_no',
            [
                'label' => esc_html__( 'Modal Title - No', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::TEXT,
                'default' => esc_html__( 'Title', 'wybmv' ),
                'placeholder' => esc_html__( 'Type your title here', 'wybmv' ),
            ]
        );
        $this->add_control(
            'modal_paragraph_no',
            [
                'label' => esc_html__( 'Modal Paragraph - No', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::TEXTAREA,
                'default' => esc_html__( 'Paragraph Text', 'wybmv' ),
                'placeholder' => esc_html__( 'Type your text here', 'wybmv' ),
            ]
        );
        $this->add_control(
            'image_no',
            [
                'label' => esc_html__( 'Choose Image - No', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::MEDIA,
                'default' => [
                    'url' => \Elementor\Utils::get_placeholder_image_src(),
                ],
            ]
        );
        $this->add_control(
            'modal_no_btn_switch',
            [
                'label' => esc_html__( 'Button Type', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::SELECT,
                'default' => 'mail',
                'options' => [
                    'call' => esc_html__( 'Phone', 'wybmv' ),
                    'mail' => esc_html__( 'Mail', 'wybmv' ),
                ]
            ]
        );
        $this->add_control(
            'modal_button_no_mail',
            [
                'label' => esc_html__( 'Add Recipient Mail', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::TEXT,
                'default' => esc_html__( 'no-reply@test.com', 'wybmv' ),
                'placeholder' => esc_html__( 'Add Recipient Mail', 'wybmv' ),
                'condition' => [
                    'modal_no_btn_switch' => 'mail',
                ],
            ]
        );
        $this->add_control(
            'modal_button_no_subject',
            [
                'label' => esc_html__( 'Add Recipient Mail Subject', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::TEXT,
                'default' => esc_html__( 'Will You Be My Valentine', 'wybmv' ),
                'placeholder' => esc_html__( 'Add Mail Subject', 'wybmv' ),
                'condition' => [
                    'modal_no_btn_switch' => 'mail',
                ],
            ]
        );
        $this->add_control(
            'modal_button_no_body',
            [
                'label' => esc_html__( 'Add Recipient Mail Message', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::TEXTAREA,
                'default' => esc_html__( 'Roses are red, WiFi is fast, my love for you is as endless as my browser history.', 'wybmv' ),
                'placeholder' => esc_html__( 'Add Mail Message', 'wybmv' ),
                'condition' => [
                    'modal_no_btn_switch' => 'mail',
                ],
            ]
        );
        $this->add_control(
            'modal_button_no_phone',
            [
                'label' => esc_html__( 'Add Recipient Phone Number', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::TEXT,
                'default' => esc_html__( '+40(261)713454', 'wybmv' ),
                'placeholder' => esc_html__( 'Add Phone Number', 'wybmv' ),
                 'condition' => [
                    'modal_no_btn_switch' => 'call',
                ],
            ]
        );
        $this->add_control(
            'modal_button_no',
            [
                'label' => esc_html__( 'Modal Button Label - No', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::TEXT,
                'default' => esc_html__( 'At least tell me why...', 'wybmv' ),
                'placeholder' => esc_html__( 'Type your button label', 'wybmv' ),
            ]
        );
        $this->end_controls_section();
        $this->start_controls_section(
            'section_modal_style',
            [
                'label' => esc_html__( 'Modal - Style', 'wybmv' ),
            ]
        );
            $this->add_group_control(
            \Elementor\Group_Control_Typography::get_type(),
            [
                'label' => esc_html__( 'Modal Title Typography', 'wybmv' ),
                'name' => 'modal_title_typography',
                'selector' => '{{WRAPPER}} .wybmv-modal-title',
            ]
        );
        $this->add_control(
            'modal_title_color',
            [
                'label' => esc_html__( 'Modal Title Color', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::COLOR,
                'default' => esc_html__( '#000', 'wybmv' ),
                'selectors' => [
                    '{{WRAPPER}} .wybmv-modal-title' => 'color: {{VALUE}}',
                ],
            ]
        );
        $this->add_group_control(
            \Elementor\Group_Control_Typography::get_type(),
            [
                'label' => esc_html__( 'Modal Description Typography', 'wybmv' ),
                'name' => 'modal_paragraph_typography',
                'selector' => '{{WRAPPER}} .wybmv-modal-paragraph',
            ]
        );
        $this->add_control(
            'modal_txt_color',
            [
                'label' => esc_html__( 'Modal Text Color', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::COLOR,
                'default' => esc_html__( '#000', 'wybmv' ),
                'selectors' => [
                    '{{WRAPPER}} .wybmv-modal-paragraph' => 'color: {{VALUE}}',
                ],
            ]
        );
        $this->add_control(
            'modal_button_yes_txt_color',
            [
                'label' => esc_html__( 'Modal Button Text Color - Yes', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::COLOR,
                'default' => esc_html__( '#000', 'wybmv' ),
                'selectors' => [
                    '{{WRAPPER}} .wybmv-modal-button.wybmv-yes' => 'color: {{VALUE}}',
                ],
            ]
        );
         $this->add_control(
            'modal_button_no_txt_color',
            [
                'label' => esc_html__( 'Modal Button Text Color - No', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::COLOR,
                'default' => esc_html__( '#000', 'wybmv' ),
                'selectors' => [
                    '{{WRAPPER}} .wybmv-modal-button.wybmv-no' => 'color: {{VALUE}}',
                ],
            ]
        );
        $this->add_control(
            'modal_button_yes_bg_color',
            [
                'label' => esc_html__( 'Modal Button Backgorund Color - Yes', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::COLOR,
                'default' => esc_html__( '#000', 'wybmv' ),
                'selectors' => [
                    '{{WRAPPER}} .wybmv-modal-button.wybmv-yes' => 'background-color: {{VALUE}}',
                ],
            ]
        );
        $this->add_control(
            'modal_button_no_bg_color',
            [
                'label' => esc_html__( 'Modal Button Backgorund Color - No', 'wybmv' ),
                'type' => \Elementor\Controls_Manager::COLOR,
                'default' => esc_html__( '#000', 'wybmv' ),
                'selectors' => [
                    '{{WRAPPER}} .wybmv-modal-button.wybmv-no' => 'background-color: {{VALUE}}',
                ],
            ]
        );
        $this->end_controls_section();
    }
    
    protected function render() {
        $settings                   = $this->get_settings_for_display();
        $shortcode_alignment        = $settings['shortcode_alignment'];
        $title                      = $settings['title'];
        $button_yes                 = $settings['button_yes'];
        $show_subtitle              = $settings['show_subtitle'];
        $subtitle                   = $settings['subtitle'];
        $button_no                  = $settings['button_no'];
        $modal_title_yes            = $settings['modal_title_yes'];
        $modal_paragraph_yes        = $settings['modal_paragraph_yes'];
        $modal_yes_btn_switch       = $settings['modal_yes_btn_switch'];
        $modal_title_no             = $settings['modal_title_no'];
        $modal_paragraph_no         = $settings['modal_paragraph_no'];
        $modal_no_btn_switch        = $settings['modal_no_btn_switch'];
        $title_color                = $settings['title_color'];
        $button_txt_color_yes       = $settings['button_txt_color_yes'];
        $button_txt_color_no        = $settings['button_txt_color_yes'];
        $button_yes_bg_color        = $settings['button_yes_bg_color'];
        $modal_title_color          = $settings['modal_title_color'];
        $modal_txt_color            = $settings['modal_txt_color'];
        $button_no_bg_color         = $settings['button_no_bg_color'];
        $image_yes                  = $settings['image_yes']['url'];
        $image_no                   = $settings['image_no']['url'];
        $modal_button_yes           = $settings['modal_button_yes'];
        $modal_button_no            = $settings['modal_button_no'];
        $modal_width                = $settings['modal_width'];
        $modal_button_yes_phone     = $settings['modal_button_yes_phone'];
        $modal_button_no_phone      = $settings['modal_button_no_phone'];
        $modal_button_no_mail       = $settings['modal_button_no_mail'];
        $modal_button_no_subject    = $settings['modal_button_no_subject'];
        $modal_button_no_body       = $settings['modal_button_no_body'];
        $modal_button_yes_mail      = $settings['modal_button_yes_mail'];
        $modal_button_yes_subject   = $settings['modal_button_yes_subject'];
        $modal_button_yes_body      = $settings['modal_button_yes_body'];
        $modal_button_yes_txt_color = $settings['modal_button_yes_txt_color'];
        $modal_button_no_txt_color  = $settings['modal_button_no_txt_color'];
        $modal_button_yes_bg_color  = $settings['modal_button_yes_bg_color'];
        $modal_button_no_bg_color   = $settings['modal_button_no_bg_color'];


        if ( ! empty( $settings['yes_url']['url'] ) ) {
            $this->add_link_attributes( 'yes_url', $settings['yes_url'] );
        }

        if ( ! empty( $settings['no_url']['url'] ) ) {
            $this->add_link_attributes( 'no_url', $settings['no_url'] );
        }

        $shortcode_content = '';
        $shortcode_content .= do_shortcode('[wybmv-builder-shortcode 
            shortcode_alignment="'.esc_attr($shortcode_alignment).'" 
            title="'.esc_attr($title).'" 
            show_subtitle="'.esc_attr($show_subtitle).'" 
            subtitle="'.esc_attr($subtitle).'" 
            button_yes="'.esc_attr($button_yes).'" 
            button_no="'.esc_attr($button_no).'" 
            title_color="'.esc_attr($title_color).'"
            button_txt_color_yes="'.esc_attr($button_txt_color_yes).'" 
            button_txt_color_yes="'.esc_attr($button_txt_color_no).'" 
            button_yes_bg_color ="'.esc_attr($button_yes_bg_color ).'" 
            button_no_bg_color="'.esc_attr($button_no_bg_color).'" 
            image_yes="'.esc_url($image_yes).'"
            image_no="'.esc_url($image_no).'"
            modal_width="'.esc_attr($modal_width).'" 
            modal_txt_color="'.esc_attr($modal_txt_color).'" 
            modal_title_color="'.esc_attr($modal_title_color).'"
            modal_title_yes="'.esc_attr($modal_title_yes).'" 
            modal_paragraph_yes="'.esc_attr($modal_paragraph_yes).'" 
            modal_title_no="'.esc_attr($modal_title_no).'" 
            modal_paragraph_no="'.esc_attr($modal_paragraph_no).'"
            modal_yes_btn_switch="'.esc_attr($modal_yes_btn_switch).'"
            modal_no_btn_switch="'.esc_attr($modal_no_btn_switch).'"
            modal_button_yes="'.esc_attr($modal_button_yes).'"
            modal_button_no="'.esc_attr($modal_button_no).'"
            modal_button_no_phone="'.esc_attr($modal_button_no_phone).'"
            modal_button_yes_phone="'.esc_attr($modal_button_yes_phone).'"
            modal_button_no_mail="'.esc_attr($modal_button_no_mail).'"
            modal_button_no_subject="'.esc_attr($modal_button_no_subject).'"
            modal_button_no_body="'.esc_attr($modal_button_no_body).'"
            modal_button_yes_mail="'.esc_attr($modal_button_yes_mail).'"
            modal_button_yes_subject="'.esc_attr($modal_button_yes_subject).'"
            modal_button_yes_body="'.esc_attr($modal_button_yes_body).'"
            modal_button_yes_txt_color="'.esc_attr($modal_button_yes_txt_color).'"
            modal_button_no_txt_color="'.esc_attr($modal_button_no_txt_color).'"
            modal_button_yes_bg_color="'.esc_attr($modal_button_yes_bg_color).'"
            modal_button_bg_bg_color="'.esc_attr($modal_button_no_bg_color).'"
        ]');

        echo  $shortcode_content;
    }
    
    protected function content_template() {

    }
}